package test;

import junit.framework.TestCase;
import compiler.*;

/**
 * <p>Title: Programming mini project 2 Java-- compiler</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2004 Nicholas Cameron</p>
 * <p>Company: </p>
 * @author Nicholas Cameron
 * @version 1.0
 */

public class TestNumberGenerator extends TestCase
{
    private NumberGenerator numGen;

    public TestNumberGenerator()
    {
    }

    public void setUp()
    {
        NumberGenerator.resetInstance();
        numGen = NumberGenerator.getInstance();
    }

    public void testLocalsEmpty()
    {
        assertEquals(11, numGen.getTotalLocals());
        assertEquals(11, numGen.getMaxLocals());
    }

    public void testLocals()
    {
        assertEquals(11, numGen.getLocal());
        assertEquals(12, numGen.getLocal());
        assertEquals(13, numGen.getTotalLocals());
        assertEquals(13, numGen.getMaxLocals());
    }

    public void testLocalsReset()
    {
        assertEquals(11, numGen.getLocal());
        assertEquals(12, numGen.getLocal());
        numGen.resetLocals();
        assertEquals(11, numGen.getTotalLocals());
        assertEquals(13, numGen.getMaxLocals());
    }

    public void testRetAddsEmpty()
    {
        assertEquals(0, numGen.getTotalRetAdds());
    }

    public void testRetAdds()
    {
        assertEquals(0, numGen.getRetAddress());
        assertEquals(1, numGen.getRetAddress());
        assertEquals(2, numGen.getTotalRetAdds());
    }

    public void testLabelsEmpty()
    {
        assertEquals(0, numGen.getTotalLabels());
    }

    public void testLabels()
    {
        String l1 = numGen.getLabel();
        String l2 = numGen.getLabel();
        assertFalse("labels should be different", l1 == l2);
        assertEquals(2, numGen.getTotalLabels());
    }

    public void testOffsets()
    {
        assertEquals(0, numGen.getOffset());
        assertEquals(1, numGen.getOffset());
    }

    public void testMethodsEmpty()
    {
        assertEquals(0, numGen.getTotalMethods());
    }

    public void testMethods()
    {
        MethodSymbolProxy m1 = new MethodSymbolProxy();
        MethodSymbolProxy m2 = new MethodSymbolProxy();
        assertEquals(0, numGen.getMethodNumber(m1));
        assertEquals(1, numGen.getMethodNumber(m2));
        assertEquals(m1, numGen.getMethod(0));
        assertEquals(m2, numGen.getMethod(1));
        assertFalse(m1 == numGen.getMethod(1));
        assertEquals(2, numGen.getTotalMethods());
    }

    public void testFields()
    {
        FieldSymbolProxy f1 = new FieldSymbolProxy();
        FieldSymbolProxy f2 = new FieldSymbolProxy();
        assertEquals(0, numGen.getFieldNumber(f1));
        assertEquals(1, numGen.getFieldNumber(f2));
        assertEquals(f1, numGen.getField(0));
        assertEquals(f2, numGen.getField(1));
        assertFalse(f1 == numGen.getField(1));
    }

    public void testMakeMethodLabel()
    {
        assertEquals("makeMethodLabel should return the same label", numGen.makeMethodLabel(0), numGen.makeMethodLabel(0));
        assertEquals("makeMethodLabel should return the same label", numGen.makeMethodLabel(16), numGen.makeMethodLabel(16));
        assertFalse("makeMethodLabel should return different labels", numGen.makeMethodLabel(16).equals(numGen.makeMethodLabel(0)));
    }

    public void testRetAdd()
    {
        assertEquals("makeRetAdd should return the same label", numGen.makeRetAdd(0), numGen.makeRetAdd(0));
        assertEquals("makeRetAdd should return the same label", numGen.makeRetAdd(16), numGen.makeRetAdd(16));
        assertFalse("makeRetAdd should return different labels", numGen.makeRetAdd(16).equals(numGen.makeRetAdd(0)));
    }

    public void testDiffLabels()
    {
        assertFalse("makeRetAdd and makeMethodLabel should return different labels", numGen.makeRetAdd(16).equals(numGen.makeMethodLabel(0)));
        assertFalse("makeRetAdd and makeMethodLabel should return different labels", numGen.makeRetAdd(16).equals(numGen.makeMethodLabel(16)));
    }
}

class MethodSymbolProxy extends MethodSymbol
{
    MethodSymbolProxy()
    {
        super(null, null, null, null, null, null, false);
    }
}

class FieldSymbolProxy extends FieldSymbol
{
    FieldSymbolProxy()
    {
        super(null, null, null, null);
    }
}
